package escape;

import java.awt.*;

public class ClosetMode extends GameMode {

  public static final int PUZZLE_REGION = 0;
  public static final int BOTTLE_REGION = 1;
  public static final int MAT_REGION = 2;
  public static final int SCREWDRIVER_REGION = 3;

  public static final int LEFT_REGION = 100;
  public static final int RIGHT_REGION = 101;
  public static final int UP_LEFT_REGION = 102;
  public static final int UP_RIGHT_REGION = 103;
  public static final int DOWN_REGION = 104;

  public int solvedCountdown;
  public boolean doorOpened;

  public ClosetMode() {
    addClickRegions();
  }

  private void addClickRegions() {
    if (doorOpened) {
      if (!GameState.acquiredBottle) {
        clickRegions.add(new Region(470, 400, 47, 80, BOTTLE_REGION));
      } else if (!GameState.liftedMat) {
        clickRegions.add(new Region(518, 440, 39, 22, MAT_REGION));
      } else if (!GameState.acquiredScrewdriver) {
        clickRegions.add(new Region(514, 446, 40, 9, SCREWDRIVER_REGION));
      }
    } else {
      clickRegions.add(new Region(529, 277, 52, 52, PUZZLE_REGION));
    }

    clickRegions.add(new Region(0, 80, 60, 520, LEFT_REGION,
        Images.arrowCursors[Images.LEFT]));
    clickRegions.add(new Region(740, 80, 60, 520, RIGHT_REGION,
        Images.arrowCursors[Images.RIGHT]));
    clickRegions.add(new Region(40, 80, 343, 100, UP_LEFT_REGION,
        Images.arrowCursors[Images.UP]));
    clickRegions.add(new Region(616, 80, 124, 100, UP_RIGHT_REGION,
        Images.arrowCursors[Images.UP]));
    clickRegions.add(new Region(40, 520, 700, 80, DOWN_REGION,
        Images.arrowCursors[Images.DOWN]));
  }

  @Override
  public void modify() {
    if (solvedCountdown > 0) {
      if (--solvedCountdown == 0) {
        doorOpened = true;
        addClickRegions();
      }
    }
  }

  @Override
  public void draw(Graphics2D g) {
    if (doorOpened) {
      g.drawImage(Images.closests[1], 0, 80, null);

      if (GameState.liftedMat) {
        if (!GameState.acquiredScrewdriver) {
          g.drawImage(Images.screwdrivers[1], 514, 446, null);
        }
        g.drawImage(Images.mats[1], 419, 440, null);
      } else {
        g.drawImage(Images.mats[0], 419, 440, null);
      }

      if (!GameState.acquiredBottle) {
        g.drawImage(Images.bottles[0], 470, 400, null);
      }

    } else {
      g.drawImage(Images.closests[0], 0, 80, null);

      for(int y = 0; y < 4; y++) {
        for(int x = 0; x < 4; x++) {
          if (main.tileMode.tiles[y][x] < 15) {
            g.drawImage(Images.smallTiles[main.tileMode.tiles[y][x]],
                529 + x * 13, 277 + y * 13, null);
          }
        }
      }

      if (solvedCountdown > 0 && ((solvedCountdown >> 3) & 1) == 0) {
        g.setColor(Color.RED);
        g.fillRect(568, 316, 13, 13);
      }
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {
    if (clickRegion.id == LEFT_REGION) {
      main.setMode(main.stallsMode);
    } else if (clickRegion.id == RIGHT_REGION) {
      if (GameState.solvedUrinalPuzzle) {
        main.setMode(main.pipesMode);
      } else {
        main.setMode(main.urinalMode);
      }
    } else if (clickRegion.id == UP_LEFT_REGION
        || clickRegion.id == UP_RIGHT_REGION) {
      main.setMode(main.smokeAlarmMode);
    } else if (clickRegion.id == DOWN_REGION) {
      main.setMode(main.drainMode);
    } else if (clickRegion.id == PUZZLE_REGION) {
      main.setMode(main.tileMode);
    } else if (clickRegion.id == BOTTLE_REGION) {
      addToInventory(Images.bottles[1]);
      GameState.acquiredBottle = true;
    } else if (clickRegion.id == MAT_REGION) {
      GameState.liftedMat = true;
    } else if (clickRegion.id == SCREWDRIVER_REGION) {
      addToInventory(Images.screwdrivers[0]);
      GameState.acquiredScrewdriver = true;
    }
    addClickRegions();
  }

  @Override
  public boolean isInventoryAccessible() {
    return solvedCountdown == 0;
  }
}
