package escape;

import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;

public final class Images {

  public static final int RIGHT = 0;
  public static final int UP = 1;
  public static final int LEFT = 2;
  public static final int DOWN = 3;

  public static Cursor blankCursor;
  public static MouseCursor[] arrowCursors = new MouseCursor[4];

  public static BufferedImage urinalBackground;
  public static BufferedImage urinalBottom;
  public static BufferedImage urinalTop1;
  public static BufferedImage urinalTop2;
  public static BufferedImage urinalWater;
  public static BufferedImage inventoryTile;
  public static BufferedImage trashCanBackground;
  public static BufferedImage pipesBackground;
  public static BufferedImage doorBackground;
  public static BufferedImage keyholeBackground;
  public static BufferedImage keyholeForeground;
  public static BufferedImage horizontalLine;
  public static BufferedImage paperTowelDispenser;
  public static BufferedImage lock;
  public static BufferedImage battery;
  public static BufferedImage drain;
  public static BufferedImage sinks;
  public static BufferedImage steam;
  public static BufferedImage tooth;
  public static BufferedImage stallsBackground;
  public static BufferedImage stallDivider;
  public static BufferedImage stall;
  public static BufferedImage keypadDisplay;
  public static BufferedImage doorHandle;
  public static BufferedImage magnetBox;
  public static BufferedImage openedDoor;
  public static BufferedImage[] ventCovers = new BufferedImage[2];
  public static BufferedImage[] arrows = new BufferedImage[4];
  public static BufferedImage[] largeVents = new BufferedImage[4];
  public static BufferedImage[] smallVents = new BufferedImage[4];
  public static BufferedImage[] ventRooms = new BufferedImage[3];
  public static BufferedImage[] characters = new BufferedImage[256];
  public static BufferedImage[] trashCans = new BufferedImage[6];
  public static BufferedImage[] tiles = new BufferedImage[15];
  public static BufferedImage[] smallTiles = new BufferedImage[15];
  public static BufferedImage[] valves = new BufferedImage[2];
  public static BufferedImage[] indicators = new BufferedImage[2];
  public static BufferedImage[] screwdrivers = new BufferedImage[2];
  public static BufferedImage[] pennies = new BufferedImage[3];
  public static BufferedImage[] keycards = new BufferedImage[3];
  public static BufferedImage[] keycardLocks = new BufferedImage[4];
  public static BufferedImage[] paperTowels = new BufferedImage[3];
  public static BufferedImage[] cranks = new BufferedImage[4];
  public static BufferedImage[] springs = new BufferedImage[5];
  public static BufferedImage[] blocks = new BufferedImage[5];
  public static BufferedImage[] picks = new BufferedImage[3];
  public static BufferedImage[] smokeAlarms = new BufferedImage[3];
  public static BufferedImage[] magnets = new BufferedImage[4];
  public static BufferedImage[] shoeLaces = new BufferedImage[4];
  public static BufferedImage[] drains = new BufferedImage[2];
  public static BufferedImage[] closests = new BufferedImage[2];
  public static BufferedImage[] bottles = new BufferedImage[3];
  public static BufferedImage[] mats = new BufferedImage[2];
  public static BufferedImage[] printouts = new BufferedImage[3];
  public static BufferedImage[] knobs = new BufferedImage[2];
  public static BufferedImage[] runningWater = new BufferedImage[2];
  public static BufferedImage[] mirrors = new BufferedImage[3];
  public static BufferedImage[] shards = new BufferedImage[3];
  public static BufferedImage[] mirrorHoles = new BufferedImage[3];
  public static BufferedImage[] stallDoors = new BufferedImage[2];
  public static BufferedImage[] stallLocks = new BufferedImage[3];
  public static BufferedImage[] toiletMonsters = new BufferedImage[4];
  public static BufferedImage[] pouring = new BufferedImage[2];
  public static BufferedImage[] shoes = new BufferedImage[6];
  public static BufferedImage[] toiletPapers = new BufferedImage[4];
  public static BufferedImage[] buttons = new BufferedImage[2];
  public static BufferedImage[] lights = new BufferedImage[3];
  public static BufferedImage[] batterySockets = new BufferedImage[2];
  public static BufferedImage[] keypads = new BufferedImage[3];
  public static BufferedImage[] plungers = new BufferedImage[2];
  public static BufferedImage[] endings = new BufferedImage[11];

  static {
    try {

      BufferedImage emptyImage = new BufferedImage(
          16, 16, BufferedImage.TYPE_INT_ARGB);
      blankCursor = Toolkit.getDefaultToolkit().createCustomCursor(
          emptyImage, new Point(0, 0), "blank cursor");

      String str = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*"
          + "()-_+=~`{}[]|\\:;\"'<>,.?/ \u00A9  ";
      BufferedImage chars = loadImage("0");
      for(int i = 0; i < 4; i++) {
        for(int j = 0; j < 18; j++) {
          characters[str.charAt(i * 18 + j)] = getSubimage(
              chars, j * 25, i * 25, 25, 25);
        }
      }
      for(int i = 0; i < 26; i++) {
        characters['a' + i] = characters['A' + i];
      }

      urinalBackground = loadImage("1");
      urinalBottom = loadImage("2");
      urinalTop1 = loadImage("3");
      urinalTop2 = loadImage("4");
      urinalWater = loadImage("5");
      inventoryTile = loadImage("6");
      arrows[RIGHT] = loadImage("7");
      arrows[LEFT] = getMirroredImage(arrows[RIGHT], true);
      arrows[DOWN] = getRotatedImage(arrows[RIGHT]);
      arrows[UP] = getMirroredImage(arrows[DOWN], false);
      largeVents[LEFT] = loadImage("8");
      largeVents[RIGHT] = getMirroredImage(largeVents[LEFT], true);
      largeVents[UP] = loadImage("9");
      largeVents[DOWN] = getMirroredImage(largeVents[UP], false);
      smallVents[LEFT] = loadImage("10");
      smallVents[RIGHT] = getMirroredImage(smallVents[LEFT], true);
      smallVents[UP] = loadImage("11");
      smallVents[DOWN] = getMirroredImage(smallVents[UP], false);
      ventRooms[0] = loadImage("12");
      ventRooms[1] = loadImage("13");
      ventRooms[2] = loadImage("14");
      trashCans[0] = loadImage("15");
      trashCans[1] = loadImage("16");
      trashCans[2] = loadImage("17");
      trashCans[3] = loadImage("18");
      trashCans[4] = loadImage("19");
      trashCans[5] = loadImage("20");
      trashCanBackground = loadImage("21");

      BufferedImage tilesImage = loadImage("22");
      for(int y = 0; y < 4; y++) {
        for(int x = 0; x < 4; x++) {
          int index = y * 4 + x;
          if (index < 15) {
            tiles[index] = getSubimage(tilesImage, x * 130, y * 130, 130, 130);
          }
        }
      }
      tilesImage = null;

      pipesBackground = loadImage("23");
      valves[0] = loadImage("24");
      valves[1] = loadImage("25");
      indicators[0] = loadImage("26");
      indicators[1] = loadImage("27");
      screwdrivers[0] = loadImage("28");
      screwdrivers[1] = loadImage("29");
      ventCovers[0] = loadImage("30");
      ventCovers[1] = loadImage("31");
      doorBackground = loadImage("32");
      pennies[0] = loadImage("33");
      pennies[1] = loadImage("34");
      keyholeForeground = loadImage("35");
      keyholeBackground = loadImage("36");
      keycards[0] = loadImage("37");
      keycards[1] = loadImage("38");
      keycardLocks[0] = loadImage("39");
      keycardLocks[1] = loadImage("40");
      keycardLocks[2] = loadImage("41");
      keycardLocks[3] = loadImage("42");
      keycards[2] = loadImage("43");
      horizontalLine = loadImage("44");
      paperTowels[0]= loadImage("45");
      paperTowelDispenser = loadImage("46");
      paperTowels[1]= loadImage("47");
      cranks[0] = loadImage("48");
      cranks[1] = loadImage("49");
      cranks[2] = loadImage("50");
      cranks[3] = loadImage("51");
      paperTowels[2]= loadImage("52");
      blocks[0] = loadImage("53");
      blocks[1] = loadImage("54");
      blocks[2] = loadImage("55");
      blocks[3] = loadImage("56");
      blocks[4] = loadImage("57");
      springs[0] = loadImage("58");
      springs[1] = loadImage("59");
      springs[2] = loadImage("60");
      springs[3] = loadImage("61");
      springs[4] = loadImage("62");
      lock = loadImage("63");
      picks[0] = loadImage("64");
      picks[1] = loadImage("65");
      picks[2] = loadImage("66");
      smokeAlarms[0] = loadImage("67");
      smokeAlarms[1] = loadImage("68");
      smokeAlarms[2] = loadImage("69");
      battery = loadImage("70");
      shoeLaces[0] = loadImage("71");
      magnets[0] = loadImage("72");
      magnets[1] = loadImage("73");
      shoeLaces[1] = loadImage("74");
      shoeLaces[2] = loadImage("75");
      magnets[2] = loadImage("76");
      magnets[3] = getRotatedImage(magnets[1]);
      shoeLaces[3] = getRotatedImage(shoeLaces[2]);
      drain = loadImage("77");
      drains[0] = loadImage("78");
      drains[1] = loadImage("79");
      closests[0] = loadImage("80");      

      BufferedImage smallTilesImage = loadImage("81");
      for(int y = 0; y < 4; y++) {
        for(int x = 0; x < 4; x++) {
          int index = y * 4 + x;
          if (index < 15) {
            smallTiles[index] = getSubimage(
                smallTilesImage, x * 13, y * 13, 13, 13);
          }
        }
      }
      smallTilesImage = null;

      bottles[0] = loadImage("82");
      mats[0] = loadImage("83");
      mats[1] = loadImage("84");
      bottles[1] = loadImage("85");
      closests[1] = loadImage("86");
      printouts[0] = loadImage("87");
      printouts[1] = loadImage("88");
      printouts[2] = loadImage("89");
      sinks = loadImage("90");
      knobs[0] = loadImage("91");
      knobs[1] = loadImage("92");
      runningWater[0] = loadImage("93");
      runningWater[1] = loadImage("94");
      mirrors[0] = loadImage("95");
      mirrors[1] = loadImage("96");
      mirrors[2] = loadImage("97");
      steam = loadImage("98");
      mirrorHoles[0] = loadImage("99");
      mirrorHoles[1] = loadImage("100");
      mirrorHoles[2] = loadImage("101");
      shards[0] = loadImage("102");
      shards[1] = loadImage("103");
      shards[2] = loadImage("104");
      tooth = loadImage("105");
      stallsBackground = loadImage("106");
      stallDoors[0] = loadImage("107");
      stallDoors[1] = loadImage("108");
      stallDivider = loadImage("109");
      stallLocks[0] = loadImage("110");
      stallLocks[1] = loadImage("111");
      pennies[2] = loadImage("112");
      stallLocks[2] = loadImage("113");
      stall = loadImage("114");
      toiletMonsters[0] = loadImage("115");
      toiletMonsters[1] = loadImage("116");
      toiletMonsters[2] = loadImage("117");
      toiletMonsters[3] = loadImage("118");
      pouring[0] = loadImage("119");
      pouring[1] = loadImage("120");
      bottles[2] = loadImage("121");
      shoes[0] = loadImage("122");
      shoes[1] = loadImage("123");
      shoes[2] = loadImage("124");
      shoes[3] = loadImage("125");
      shoes[4] = loadImage("126");
      shoes[5] = loadImage("127");
      toiletPapers[0] = loadImage("128");
      toiletPapers[1] = loadImage("129");
      toiletPapers[2] = loadImage("130");
      toiletPapers[3] = loadImage("131");
      buttons[0] = loadImage("132");
      buttons[1] = loadImage("133");
      lights[0] = loadImage("134");
      lights[1] = loadImage("135");
      lights[2] = loadImage("136");
      batterySockets[0] = loadImage("137");
      batterySockets[1] = loadImage("138");
      keypadDisplay = loadImage("139");
      keypads[0] = loadImage("140");
      keypads[1] = loadImage("141");
      keypads[2] = loadImage("142");
      plungers[0] = loadImage("143");
      plungers[1] = loadImage("144");
      doorHandle = loadImage("145");
      magnetBox = loadImage("146");
      endings[9] = loadImage("147");
      endings[8] = loadImage("148");
      endings[7] = loadImage("149");
      endings[6] = loadImage("150");
      endings[5] = loadImage("151");
      endings[4] = loadImage("152");
      endings[3] = loadImage("153");
      endings[2] = loadImage("154");
      endings[1] = loadImage("155");
      endings[0] = loadImage("156");
      endings[10] = loadImage("157");
      openedDoor = loadImage("158");

      arrowCursors[RIGHT] = new MouseCursor(arrows[RIGHT], 41, 19);
      arrowCursors[LEFT] = new MouseCursor(arrows[LEFT], 1, 19); 
      arrowCursors[UP] = new MouseCursor(arrows[UP], 19, 1);
      arrowCursors[DOWN] = new MouseCursor(arrows[DOWN], 19, 41);

    } catch(Throwable t) {
      t.printStackTrace();
    }
  }

  private static BufferedImage getRotatedImage(BufferedImage image) {
    BufferedImage image2 = new BufferedImage(
        image.getHeight(), image.getWidth(), BufferedImage.TYPE_4BYTE_ABGR_PRE);
    int[] pixels = new int[image.getWidth() * image.getHeight()];
    image.getRGB(0, 0, image.getWidth(), image.getHeight(),
        pixels, 0, image.getWidth());
    int[] pixels2 = new int[image.getWidth() * image.getHeight()];
    for(int y = 0; y < image.getHeight(); y++) {
      for(int x = 0; x < image.getWidth(); x++) {
        pixels2[x * image.getHeight() + y] = pixels[y * image.getWidth() + x];
      }
    }
    image2.setRGB(0, 0, image2.getWidth(), image2.getHeight(),
        pixels2, 0, image2.getWidth());
    return image2;
  }

  private static BufferedImage getMirroredImage(
      BufferedImage image, boolean leftRight) {

    BufferedImage image2 = new BufferedImage(
        image.getWidth(), image.getHeight(), BufferedImage.TYPE_4BYTE_ABGR_PRE);
    Graphics2D g = (Graphics2D)image2.getGraphics();
    if (leftRight) {
      g.drawImage(image, image.getWidth(), 0, 0, image.getHeight(),
          0, 0, image.getWidth(), image.getHeight(), null);
    } else {
      g.drawImage(image, 0, image.getHeight(), image.getWidth(), 0,
          0, 0, image.getWidth(), image.getHeight(), null);
    }
    g.dispose();
    return image2;
  }

  private static BufferedImage getSubimage(
      BufferedImage image, int x, int y, int width, int height) {
    BufferedImage subimage = new BufferedImage(
        width, height, BufferedImage.TYPE_4BYTE_ABGR_PRE);
    Graphics2D g = (Graphics2D)subimage.getGraphics();
    g.drawImage(image, 0, 0, width, height, x, y, x + width, y + height, null);
    g.dispose();
    return subimage;
  }

  private static BufferedImage loadImage(String name) throws Throwable {
    return ImageIO.read(Images.class.getResource("/images/" + name + ".png"));
  }

  private Images() {
  }

  public static void drawString(Graphics g, String string, int x, int y) {
    for(int i = 0; i < string.length(); i++) {
      g.drawImage(characters[string.charAt(i)], x, y, null);
      x += 25;
    }
  }

  public static void drawString(Graphics g, CharSequence string, int x, int y) {
    for(int i = 0; i < string.length(); i++) {
      g.drawImage(characters[string.charAt(i)], x, y, null);
      x += 25;
    }
  }
}
