package escape;

import java.awt.*;
import java.awt.image.BufferedImage;

public class KeypadMode extends GameMode {

  public static final int REGION_BATTERY = 100;
  public static final int REGION_EXIT = 101;

  public static final String[] numbers
      = { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" };

  private int buttonDown = -1;
  private int buttonDownDelay;
  private String password = "";
  private int passwordEnteredDelay = 60;

  public KeypadMode() {
    addClickRegions();
  }

  private void addClickRegions() {
    clickRegions.clear();
    if (!GameState.passwordEntered) {
      if (buttonDown == -1 && buttonDownDelay == 0) {
        for(int y = 0; y < 3; y++) {
          for(int x = 0; x < 3; x++) {
            clickRegions.add(new Region(313 + 60 * x, 275 + 50 * y, 54, 47,
                1 + y * 3 + x));
          }
        }
        clickRegions.add(new Region(373, 425, 54, 47, 0));
      }
      clickRegions.add(new Region(0, 80, 200, 520,
          REGION_EXIT, Images.arrowCursors[Images.DOWN]));
      clickRegions.add(new Region(600, 80, 200, 520,
          REGION_EXIT, Images.arrowCursors[Images.DOWN]));
    }
  }

  @Override
  public void modify() {
    if (buttonDownDelay > 0) {
      if (--buttonDownDelay == 0) {
        buttonDown = -1;
        addClickRegions();
      }
    }
    if (GameState.passwordEntered && --passwordEnteredDelay == 0) {
      main.popMode();
    }
  }

  @Override
  public void draw(Graphics2D g) {
    g.setColor(Color.WHITE);
    g.fillRect(0, 80, 800, 520);

    for(int y = 0; y < 3; y++) {
      for(int x = 0; x < 3; x++) {
        int index = 1 + y * 3 + x;
        if (index == buttonDown) {
          g.drawImage(Images.buttons[1], 313 + 60 * x, 275 + 50 * y, null);
          Images.drawString(g, numbers[index],
              330 + 60 * x, 285 + 50 * y);
        } else {
          g.drawImage(Images.buttons[0], 313 + 60 * x, 275 + 50 * y, null);
          Images.drawString(g, numbers[index],
              323 + 60 * x, 280 + 50 * y);
        }
      }
    }
    if (buttonDown == 0) {
      g.drawImage(Images.buttons[1], 373, 425, null);
      Images.drawString(g, "0", 390, 435);
    } else {
      g.drawImage(Images.buttons[0], 373, 425, null);
      Images.drawString(g, "0", 383, 430);
    }

    g.drawImage(Images.lights[
        GameState.batteryInstalled ? GameState.passwordEntered ? 2 : 1 : 0],
            333, 106, null);
    g.drawImage(Images.keypadDisplay, 290, 226, null);
    g.drawImage(Images.batterySockets[GameState.batteryInstalled ? 0 : 1],
        322, 486, null);

    Images.drawString(g, password, 300, 230);
  }

  @Override
  public void inventoryItemLifted(BufferedImage image) {
    if (image == Images.battery) {
      dropRegions.add(new Region(340, 496, 125, 65, REGION_BATTERY));
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {
    if (clickRegion.id <= 9) {
      buttonDownDelay = 10;
      buttonDown = clickRegion.id;
      if (GameState.batteryInstalled && !GameState.passwordEntered
        && password.length() < 8) {
          password += numbers[buttonDown];
        if (password.equals(GameState.entryCode)) {
          GameState.passwordEntered = true;
        }
      }
    } else if (clickRegion.id == REGION_EXIT) {
      main.popMode();
    }
  }

  @Override
  public void regionDropped(BufferedImage image, Region dropRegion) {
    if (image == Images.battery) {
      GameState.batteryInstalled = true;
    }
  }

  @Override
  public boolean isInventoryAccessible() {
    return buttonDown == -1 && buttonDownDelay == 0;
  }

  @Override
  public void enter() {
    super.enter();
    addClickRegions();
    if (!GameState.passwordEntered) {
      password = "";
    }
  }
}
