package escape;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.*;

public class MirrorMode extends GameMode {

  public static final int STATE_START = 0;
  public static final int STATE_BLUE_DOWN = 1;
  public static final int STATE_RED_DOWN = 2;
  public static final int STATE_SMASHING = 3;

  public static final int REGION_RED = 0;
  public static final int REGION_BLUE = 1;
  public static final int REGION_LEFT = 2;
  public static final int REGION_RIGHT = 3;
  public static final int REGION_TOOTH = 4;
  public static final int REGION_HOLE = 5;

  private int state = STATE_START;
  private float redY = 389;
  private float blueY = 389;
  private int runningWaterIndex;
  private int runningWaterIndexIncrementor;
  private boolean hiddenMessageMirror;
  private boolean fogVisible;
  private int steamIncrementor;

  private ArrayList<Steam> steams = new ArrayList<Steam>();
  private ArrayList<Shard> shards = new ArrayList<Shard>();

  private void addClickRegions() {
    if (shards.size() == 0 && steams.size() == 0 && state == STATE_START) {
      clickRegions.clear();
      clickRegions.add(new Region(314, 389, 38, 23, REGION_RED));
      clickRegions.add(new Region(439, 389, 38, 23, REGION_BLUE));      
      clickRegions.add(new Region(0, 80, 200, 520, REGION_LEFT,
          Images.arrowCursors[Images.LEFT]));
      clickRegions.add(new Region(600, 80, 200, 520, REGION_RIGHT,
          Images.arrowCursors[Images.RIGHT]));

      if (hiddenMessageMirror && GameState.shatteredMirror) {
        clickRegions.add(new Region(305, 98, 70, 65, REGION_HOLE));
      }
    }
  }

  @Override
  public void modify() {

    if (++runningWaterIndexIncrementor == 7) {
      runningWaterIndexIncrementor = 0;
      if (++runningWaterIndex == 2) {
        runningWaterIndex = 0;
      }
    }

    if (state == STATE_SMASHING) {
      for(int i = shards.size() - 1; i >= 0; i--) {
        Shard shard = shards.get(i);
        if (!shard.move()) {
          shards.remove(i);
        }
      }
      if (shards.size() == 0) {
        state = STATE_START;
        addClickRegions();
      }
    } else if (state == STATE_BLUE_DOWN) {
      blueY -= 0.05556f;
      if (blueY <= 389) {
        blueY = 389;
        state = STATE_START;
        addClickRegions();
      }
    } else if (state == STATE_RED_DOWN) {
      if (redY > 392 && --steamIncrementor < 0) {
        steamIncrementor = 20;
        steams.add(new Steam(
            366 + random.nextInt(10), 512 + random.nextInt(10)));
      }
      redY -= 0.02778f;
      if (redY <= 389) {
        redY = 389;
        if (steams.size() == 0) {
          state = STATE_START;
          addClickRegions();          
        }
      } else if (redY <= 390) {
        fogVisible = true;
      }

      for(int i = steams.size() - 1; i >= 0; i--) {
        Steam steam = steams.get(i);
        steam.y -= 2;
        if (steam.y < 355) {
          steam.x += steam.vx;
        }
        if (steam.y < -82) {
          steams.remove(i);
        }
      }
    }
  }

  @Override
  public void draw(Graphics2D g) {
    g.drawImage(Images.mirrors[fogVisible ? hiddenMessageMirror ? 2 : 1 : 0],
        0, 80, null);

    if (hiddenMessageMirror && GameState.shatteredMirror) {
      g.drawImage(Images.mirrorHoles[1], 295, 98, null);
    }
    if (state == STATE_SMASHING) {
      for(int i = shards.size() - 1; i >= 0; i--) {
        Shard shard = shards.get(i);
        g.drawImage(Images.shards[shard.index],
            (int)shard.x, (int)shard.y, null);
      }
    }

    g.drawImage(Images.knobs[0], 314, (int)redY, null);
    g.drawImage(Images.knobs[1], 439, (int)blueY, null);

    if (state == STATE_BLUE_DOWN 
        || (state == STATE_RED_DOWN && redY > 389)) {
      g.drawImage(Images.runningWater[runningWaterIndex], 353, 444, null);
    }

    for(int i = steams.size() - 1; i >= 0; i--) {
      Steam steam = steams.get(i);
      int y = (int)steam.y;
      int height = 162;
      if (y < 80) {
        height = 82 + y;
        y = 80;
      }
      if (y > 355) {
        height -= y - 355;
      }
      if (height > 0) {
        g.setClip((int)steam.x, y, 49, height);
        y = 162 * (((int)steam.y) / 162);
        g.drawImage(Images.steam, (int)steam.x, y, null);
        g.drawImage(Images.steam, (int)steam.x, y + 162, null);
        g.setClip(null);
      }
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {
    if (clickRegion.id == REGION_HOLE) {
      main.pushMode(main.mirrorHoleMode);
    } else if (clickRegion.id == REGION_RED) {
      state = STATE_RED_DOWN;
      redY += 10;      
    } else if (clickRegion.id == REGION_BLUE) {
      state = STATE_BLUE_DOWN;
      blueY += 10;
    } else if (clickRegion.id == REGION_LEFT
        || clickRegion.id == REGION_RIGHT) {
      main.setMode(main.sinksMode);
    }
  }

  @Override
  public void regionDropped(BufferedImage image, Region dropRegion) {
    if (image == Images.tooth) {
      clickRegions.clear();
      state = STATE_SMASHING;
      GameState.shatteredMirror = true;

      shards.add(new Shard(295, 98, -3, -6, 0));
      shards.add(new Shard(295, 98, 3, -5, 1));
      shards.add(new Shard(295, 98, 1, -4, 2));
    }
  }

  @Override
  public void inventoryItemLifted(BufferedImage image) {
    if (fogVisible && hiddenMessageMirror && image == Images.tooth) {
      dropRegions.add(new Region(301, 95, 83, 69, REGION_TOOTH));
    }
  }

  @Override
  public void enter() {
    super.enter();
    fogVisible = false;
    addClickRegions();
    state = STATE_START;
    redY = 389;
    blueY = 389;
  }

  @Override
  public boolean isInventoryAccessible() {
    return state == STATE_START;
  }

  public boolean isHiddenMessageMirror() {
    return hiddenMessageMirror;
  }

  public void setHiddenMessageMirror(boolean hiddenMessageMirror) {
    this.hiddenMessageMirror = hiddenMessageMirror;
  }
}
