package escape;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.*;

public class PipesMode extends GameMode {

  public static final int PRINTOUT_REGION = 100;

  public static final int LEFT_REGION = 200;
  public static final int RIGHT_REGION = 201;
  public static final int UP_REGION = 202;
  public static final int DOWN_REGION = 203;  

  public int[][] coordinates = {
    { 72, 29 },
    { 369, 29 },
    { 660, 31 },
    { 244, 107 },
    { 88, 184 },
    { 371, 185 },
    { 559, 261 },
    { 674, 326 },
  };
  public int[] valveStates = new int[8];
  public int[] valveValues = new int[8];

  public int goal = (random.nextInt(226) + 15) << 1;
  public int current = 0;
  public int target = 0;
  public int floorY = 561;

  private boolean goalAchieved;
  private int flashing;
  private int printingY;
  private boolean printoutAvailable;

  public PipesMode() {
    for(int i = 0; i < 8; i++) {
      coordinates[i][1] += 80;
    }
    ArrayList<Integer> list = new ArrayList<Integer>();
    for(int i = 1; i <= 128; i <<= 1) {
      list.add(i);
    }
    for(int i = 0; i < 8; i++) {
      valveValues[i] = list.remove(random.nextInt(list.size()));
    }
  }

  private void addClickRegions() {

    clickRegions.clear();

    if (!GameState.acquiredPrintout) {
      if (goalAchieved) {
        if (printoutAvailable) {
          clickRegions.add(new Region(77, 538, 27, 46, PRINTOUT_REGION));
        }
      } else if (printingY == 0 || printingY == 92) {
        for(int i = 0; i < 8; i++) {
          if (valveStates[i] == 0) {
            clickRegions.add(new Region(coordinates[i][0], coordinates[i][1],
                62, 26, i));
          } else {
            clickRegions.add(new Region(coordinates[i][0] + 39, coordinates[i][1],
                26, 64, i));
          }
        }
      }
    }

    clickRegions.add(new Region(0, 80, 32, 520, LEFT_REGION,
        Images.arrowCursors[Images.LEFT]));
    clickRegions.add(new Region(750, 80, 50, 520, RIGHT_REGION,
        Images.arrowCursors[Images.RIGHT]));
    clickRegions.add(new Region(40, 80, 718, 20, UP_REGION,
        Images.arrowCursors[Images.UP]));

    if (!GameState.acquiredPrintout && goalAchieved && printoutAvailable) {
      clickRegions.add(new Region(170, 565, 588, 35, DOWN_REGION,
          Images.arrowCursors[Images.DOWN]));
    } else {
      clickRegions.add(new Region(40, 565, 718, 35, DOWN_REGION,
          Images.arrowCursors[Images.DOWN]));
    }
  }

  @Override
  public void modify() {
    if (!GameState.acquiredPrintout) {
      if (goalAchieved) {
        flashing++;
        if (printingY < 92) {
          printingY++;
        } else {
          printoutAvailable = true;
          addClickRegions();
        }
      } else if (floorY >= 75) {
        floorY -= 2;
        if (floorY < 75) {
          addClickRegions();
        }
      } else if (GameState.keycardUsed) {

        if (current != target) {
          if (current < target) {
            current++;
          } else {
            current--;
          }
        } else if (current == goal) {
          goalAchieved = true;
          clickRegions.clear();
        }
      }
    }
  }

  @Override
  public void draw(Graphics2D g) {
    g.drawImage(Images.pipesBackground, 0, 80, null);
    for(int i = 0; i < 8; i++) {
      g.drawImage(Images.valves[valveStates[i]],
          coordinates[i][0], coordinates[i][1], null);
    }
    if (GameState.acquiredPrintout) {
      g.drawImage(Images.keycardLocks[0], 42, 485, null);
    } else if (goalAchieved) {
      g.drawImage(Images.keycardLocks[(flashing >> 3) & 1],
          42, 485, null);
      g.setClip(70, 538, 40, 50);
      g.drawImage(Images.printouts[0], 77, 492 + (printingY >> 1), null);
      g.setClip(null);
    } else {
      g.drawImage(Images.keycardLocks[GameState.keycardUsed ? 0 : 1],
          42, 485, null);
    }
    g.drawImage(Images.indicators[1], 139 + goal, 528, null);
    g.drawImage(Images.indicators[0], 147 + current, 528, null);

    if (floorY >= 0) {
      g.setClip(35, 80, 765, 520);
      g.setColor(Color.WHITE);
      g.fillRect(35, 80, 765, floorY - 81);
      g.drawImage(Images.horizontalLine, 34, floorY, null);

      for(int i = 0; i < 5; i++) {
        g.drawImage(Images.urinalBottom, 55 + i * 150, floorY - 261, null);
        g.drawImage(Images.urinalTop1, 55 + i * 150, floorY - 304, null);
      }
      g.setClip(null);
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {

    if (clickRegion.id == LEFT_REGION) {
      main.setMode(main.closetMode);
    } else if (clickRegion.id == RIGHT_REGION) {
      main.setMode(main.sinksMode);
    } else if (clickRegion.id == UP_REGION) {
      main.setMode(main.smokeAlarmMode);
    } else if (clickRegion.id == DOWN_REGION) {
      main.setMode(main.drainMode);
    } else if (clickRegion.id == PRINTOUT_REGION) {
      addToInventory(Images.printouts[1]);
      GameState.acquiredPrintout = true;
    } else {
      valveStates[clickRegion.id] ^= 1;
      target = 0;
      for(int i = 0; i < 8; i++) {
        target += valveStates[i] * valveValues[i];
      }
      target <<= 1;
    }
    addClickRegions();
  }

  @Override
  public void inventoryItemLifted(BufferedImage image) {
    if (image == Images.keycards[0]) {
      dropRegions.add(new Region(47, 490, 88, 65));
    }
  }

  @Override
  public void regionDropped(BufferedImage image, Region dropRegion) {
    Main.main.setMode(Main.main.keycardMode);
  }

  @Override
  public boolean isInventoryAccessible() {
    return floorY < 75 && (printingY == 0 || printingY == 92);
  }
}
