package escape;

import java.awt.*;
import java.awt.image.*;
import java.util.*;

public class TrashCanMode extends GameMode {

  public static final int LEFT_REGION = 100;
  public static final int RIGHT_REGION = 101;
  public static final int UP_REGION = 102;
  public static final int DOWN_REGION = 103;

  public Stack<Integer>[] stacks = new Stack[3];
  public int lifting = -1;
  public int sourceStack;

  public TrashCanMode() {
    for(int i = 0; i < 3; i++) {
      stacks[i] = new Stack<>();
    }
    for(int i = 5; i >= 0; i--) {
      stacks[0].push(i);
    }

    addLifingClickRegions();
  }

  private void addLifingClickRegions() {
    clickRegions.clear();
    dropRegions.clear();
    for(int i = 0; i < 3; i++) {
      if (stacks[i].size() > 0) {
        clickRegions.add(new Region(100 + i * 200, 300, 195, 291, i));
      }
    }
    if (GameState.ventCoverRemoved && stacks[0].isEmpty()) {
      clickRegions.add(new Region(125, 390, 150, 98, 10,
          Images.arrowCursors[Images.UP]));
    }

    clickRegions.add(new Region(0, 80, 40, 520, LEFT_REGION,
        Images.arrowCursors[Images.LEFT]));
    clickRegions.add(new Region(740, 80, 60, 520, RIGHT_REGION,
        Images.arrowCursors[Images.RIGHT]));
    clickRegions.add(new Region(40, 80, 700, 100, UP_REGION,
        Images.arrowCursors[Images.UP]));
    clickRegions.add(new Region(40, 580, 700, 20, DOWN_REGION,
        Images.arrowCursors[Images.DOWN]));
  }

  private void addDropClickRegions() {
    clickRegions.clear();
    dropRegions.clear();
    for(int i = 0; i < 3; i++) {
      if (stacks[i].isEmpty()
          || stacks[i].peek() > lifting) {
        dropRegions.add(new Region(110 + i * 200, 310, 175, 271, i));
      }
    }
  }

  @Override public void modify() {
  }

  @Override public void draw(final Graphics2D g) {
    g.drawImage(Images.trashCanBackground, 0, 80, null);

    if (GameState.ventCoverRemoved) {
      g.drawImage(Images.smallVents[Images.RIGHT], 125, 390, null);
      g.drawImage(Images.ventCovers[1], 275, 425, null);
    } else {
      g.drawImage(Images.ventCovers[0], 125, 390, null);
    }

    if (stacks[0].size() > 0) {
      g.drawImage(Images.trashCans[stacks[0].peek()], 100, 300, null);
    }
    if (stacks[1].size() > 0) {
      g.drawImage(Images.trashCans[stacks[1].peek()], 300, 300, null);
    }
    if (stacks[2].size() > 0) {
      g.drawImage(Images.trashCans[stacks[2].peek()], 500, 300, null);
    }
  }

  @Override public void regionClicked(final Region clickRegion) {
    switch(clickRegion.id) {
      case LEFT_REGION:
        main.setMode(main.sinksMode);
        addLifingClickRegions();
        break;
      case RIGHT_REGION:
        main.setMode(main.doorMode);
        addLifingClickRegions();
        break;
      case UP_REGION:
        main.setMode(main.smokeAlarmMode);
        addLifingClickRegions();
        break;
      case DOWN_REGION:
        main.setMode(main.drainMode);
        addLifingClickRegions();
        break;
      case 10:
        main.ventMode.reset();
        main.setMode(main.ventMode);
        addLifingClickRegions();
        break;
      default:
        sourceStack = clickRegion.id;
        lifting = stacks[sourceStack].pop();
        dragImage(Images.trashCans[lifting], 
            Mouse.x - (100 + sourceStack * 200), Mouse.y - 300);
        addDropClickRegions();
        break;
    }
  }

  @Override public void regionDropped(final BufferedImage image, 
      final Region dropRegion) {
    
    if (image == Images.trashCans[0]
        || image == Images.trashCans[1]
        || image == Images.trashCans[2]
        || image == Images.trashCans[3]
        || image == Images.trashCans[4]
        || image == Images.trashCans[5]) {
      if (dropRegion == null) {
        stacks[sourceStack].push(lifting);    
      } else {
        stacks[dropRegion.id].push(lifting);
      }
      lifting = -1;
      addLifingClickRegions();
    } else if (image == Images.screwdrivers[0]) {
      GameState.ventCoverRemoved = true;
      addLifingClickRegions();
    }
  }

  @Override public void inventoryItemLifted(final BufferedImage image) {
    if (image == Images.screwdrivers[0]
        && !GameState.ventCoverRemoved && stacks[0].isEmpty()) {
      dropRegions.add(new Region(130, 395, 140, 88));
    }
  }
}