package escape;

import java.awt.*;
import java.util.*;

public class UrinalMode extends GameMode {

  public static final int SEQUENCE_LENGTH = 8;

  public static final int LEFT_REGION = 100;
  public static final int RIGHT_REGION = 101;
  public static final int UP_REGION = 102;
  public static final int DOWN_REGION = 103;
  
  public static final int STATE_USER_CHOOSING = 0;
  public static final int STATE_FLUSHING = 1;
  public static final int STATE_PLAYING_SEQUENCE = 2;
  public static final int STATE_BLINKING = 3;

  public static final int FLUSH_STATE_HANDLE_DOWN = 0;
  public static final int FLUSH_STATE_WATER_POURING = 1;
  public static final int FLUSH_STATE_WATER_FADING = 2;

  public Region[] handleRegions = new Region[5];
  public int state = STATE_PLAYING_SEQUENCE;
  public int flushState;
  public boolean userFlushed;
  public int urinalIndex;
  public int delay = 30;
  public ArrayList<Integer> sequence = new ArrayList<Integer>();
  public int sequenceIndex;
  public int sequenceLength;
  public boolean failed;
  public int blinkDelay = 10;

  public UrinalMode() {    
    randomizeSequence();
  }

  private void reset() {
    clickRegions.clear();
    randomizeSequence();
    state = STATE_PLAYING_SEQUENCE;
    delay = 30;
    blinkDelay = 10;
    failed = false;
    flushState = 0;
    urinalIndex = 0;
    sequenceIndex = 0;
    userFlushed = false;
  }

  public void randomizeSequence() {
    sequence.clear();
    for(int i = 0; i < SEQUENCE_LENGTH; i++) {
      sequence.add(random.nextInt(5));
    }
    sequenceLength = 1;
  }

  private void addClickRegions() {
    for(int i = 0; i < 5; i++) {
      clickRegions.add(new Region(66 + i * 150, 266, 30, 30, i));
    }

    clickRegions.add(new Region(0, 80, 40, 520, LEFT_REGION,
        Images.arrowCursors[Images.LEFT]));
    clickRegions.add(new Region(740, 80, 60, 520, RIGHT_REGION,
        Images.arrowCursors[Images.RIGHT]));
    clickRegions.add(new Region(40, 80, 700, 100, UP_REGION,
        Images.arrowCursors[Images.UP]));
    clickRegions.add(new Region(40, 520, 700, 80, DOWN_REGION,
        Images.arrowCursors[Images.DOWN]));
  }

  @Override
  public void modify() {
    if (state == STATE_FLUSHING) {
      if (flushState == FLUSH_STATE_HANDLE_DOWN) {
        if (--delay == 0) {
          flushState = FLUSH_STATE_WATER_POURING;
          delay = 0;
        }
      } else if (flushState == FLUSH_STATE_WATER_POURING) {
        delay += 3;
        if (delay >= 128) {
          delay = 0;
          flushState = FLUSH_STATE_WATER_FADING;
        }
      } else if (flushState == FLUSH_STATE_WATER_FADING) {
        delay += 8;
        if (delay >= 255) {
          if (userFlushed) {
            if (failed) {
              state = STATE_USER_CHOOSING;
              addClickRegions();
            } else if (urinalIndex == sequence.get(sequenceIndex)) {
              if (++sequenceIndex == sequenceLength) {
                state = STATE_PLAYING_SEQUENCE;
                delay = 30;
                sequenceIndex = 0;
                if (sequenceLength++ == SEQUENCE_LENGTH) {
                  delay = 15;
                  state = STATE_BLINKING;
                  urinalIndex = sequence.get(sequenceIndex);
                }
              } else {
                state = STATE_USER_CHOOSING;
                addClickRegions();
              }
            } else {
              failed = true;
              state = STATE_USER_CHOOSING;
              addClickRegions();
            }
          } else {
            if (sequenceIndex == sequenceLength) {
              sequenceIndex = 0;
              state = STATE_USER_CHOOSING;
              addClickRegions();
            } else {
              state = STATE_PLAYING_SEQUENCE;
              delay = 0;
            }
          }
        }
      }
    } else if (state == STATE_PLAYING_SEQUENCE) {
      if (delay > 0) {
        delay--;
      } else {
        userFlushed = false;
        state = STATE_FLUSHING;
        flushState = FLUSH_STATE_HANDLE_DOWN;
        urinalIndex = sequence.get(sequenceIndex++);
        delay = 15;
      }
    } else if (state == STATE_BLINKING) {
      if (delay > 0) {
        delay--;
      } else {
        delay = blinkDelay;
        urinalIndex = sequence.get(sequenceIndex++);
        if (sequenceIndex == sequence.size()) {
          sequenceIndex = 0;
          if (--blinkDelay == 0) {
            GameState.solvedUrinalPuzzle = true;
            Main.main.setMode(Main.main.pipesMode);
          }
        }
      }
    }
  }

  @Override
  public void draw(Graphics2D g) {
    g.drawImage(Images.urinalBackground, 0, 80, null);
    for(int i = 0; i < 5; i++) {

      if (state == STATE_FLUSHING && i == urinalIndex) {
        if (flushState == FLUSH_STATE_WATER_POURING) {
          g.setClip(74 + i * 150, 331, 81, 107);
          g.drawImage(Images.urinalWater, 74 + i * 150, 202 + delay, null);
          g.setClip(null);
        } else if (flushState == FLUSH_STATE_WATER_FADING) {
          g.drawImage(Images.urinalWater, 74 + i * 150, 331, null);
          g.setColor(new Color(255, 255, 255, delay));
          g.fillRect(74 + i * 150, 331, 81, 107);
        }
      } else if (state == STATE_BLINKING && i == urinalIndex) {
        g.setColor(Color.CYAN);
        g.fillRect(74 + i * 150, 331, 81, 107);
      }

      g.drawImage(Images.urinalBottom, 55 + i * 150, 300, null);

      g.drawImage(state == STATE_FLUSHING
          && flushState == FLUSH_STATE_HANDLE_DOWN
          && i == urinalIndex ? Images.urinalTop2
              : Images.urinalTop1, 55 + i * 150, 257, null);
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {

    if (clickRegion.id == LEFT_REGION) {
      addClickRegions();
      main.setMode(main.closetMode);
    } else if (clickRegion.id == RIGHT_REGION) {
      addClickRegions();
      main.setMode(main.sinksMode);
    } else if (clickRegion.id == UP_REGION) {
      addClickRegions();
      main.setMode(main.smokeAlarmMode);
    } else if (clickRegion.id == DOWN_REGION) {
      addClickRegions();
      main.setMode(main.drainMode);
    } else {
      state = STATE_FLUSHING;
      flushState = FLUSH_STATE_HANDLE_DOWN;
      userFlushed = true;
      urinalIndex = clickRegion.id;
      delay = 15;
    }
  }

  @Override
  public void enter() {
    super.enter();
    reset();
  }

  @Override
  public boolean isInventoryAccessible() {
    return state == STATE_USER_CHOOSING;
  }
}
